@@ -36,12 +36,12 @@ class OptimizationProblem(Observable, SearchParam):
 
         for idx, condition in enumerate(conditions):
             if condition in exp_dict.keys():
-                sim_val.extend(sim_matrix[list(map(int, exp_timepoint)), idx])
+                sim_val.extend(sim_matrix[idx, list(map(int, exp_timepoint))])
                 exp_val.extend(exp_dict[condition])
 
         return np.array(sim_val) / sim_norm_max, np.array(exp_val)
 
-    def objective(self, indiv, *args) -> float:
+    def objective(self, indiv, *args):
         """Define an objective function to be minimized"""
         if len(args) == 0:
             (x, y0) = self.update(indiv)
@@ -70,7 +70,6 @@ class OptimizationProblem(Observable, SearchParam):
                                 np.max(
                                     self.simulations[
                                         self.obs_names.index(obs_name),
-                                        self.normalization[obs_name]["timepoint"],
                                         [
                                             self.conditions.index(c)
                                             for c in (
@@ -79,13 +78,13 @@ class OptimizationProblem(Observable, SearchParam):
                                                 else self.conditions
                                             )
                                         ],
+                                        self.normalization[obs_name]["timepoint"],
                                     ]
                                 )
                                 if self.normalization[obs_name]["timepoint"] is not None
                                 else np.max(
                                     self.simulations[
                                         self.obs_names.index(obs_name),
-                                        :,
                                         [
                                             self.conditions.index(c)
                                             for c in (
