@@ -25,6 +25,9 @@ class Text2Model(ReactionRules):
     input_txt : str
         Model description file (*.txt), e.g., 'Kholodenko1999.txt'
 
+    similarity_threshold : float (default: 0.7)
+        Similarity threshold used in text-to-model conversion. Must lie within (0, 1).
+
     lang : Literal["python", "julia"] (default: 'python')
         Either 'python' or 'julia'.
 
@@ -33,6 +36,7 @@ class Text2Model(ReactionRules):
     """
 
     input_txt: str
+    similarity_threshold: float = 0.7
     lang: Literal["python", "julia"] = "python"
     indentation: str = field(default=4 * " ", init=False)
 
@@ -157,8 +161,11 @@ class Text2Model(ReactionRules):
             ) as f:
                 f.write("\n".join(lines))
 
-    def _update_set_model(self) -> None:
-        """Update set_model.py
+    def _update_diffeq(self) -> None:
+        """Update differential equations.
+
+        * Set flux vector (v).
+        * Set rhs of ODE.
 
         If you don't write info., all parameter values and initial values are
         initialized to 1.0 and 0.0, respectively.
@@ -179,14 +186,7 @@ class Text2Model(ReactionRules):
                             f"dydt[V.{species}] = 0 # ",
                         )
             for line_num, line in enumerate(lines):
-                if line.startswith(2 * self.indentation + "v = {}\n"):
-                    # Write flux vector: v
-                    lines[line_num + 1] = (
-                        2 * self.indentation
-                        + f"\n{2 * self.indentation}".join(self.reactions)
-                        + "\n\n"
-                    )
-                elif line.startswith(2 * self.indentation + "dydt = [0] * V.NUM\n"):
+                if line.startswith(2 * self.indentation + "dydt = [0] * V.NUM\n"):
                     # Write right-hand side of the differential equation: dydt
                     lines[line_num + 1] = (
                         2 * self.indentation
@@ -225,6 +225,26 @@ class Text2Model(ReactionRules):
                 mode="w",
             ) as f:
                 f.writelines(lines)
+            with open(
+                os.path.join(os.path.dirname(__file__), "template", "reaction_network.py"),
+                encoding="utf-8",
+                mode="r",
+            ) as f:
+                lines = f.readlines()
+            for line_num, line in enumerate(lines):
+                if line.startswith(2 * self.indentation + "v = {}\n"):
+                    # Write flux vector: v
+                    lines[line_num + 1] = (
+                        2 * self.indentation
+                        + f"\n{2 * self.indentation}".join(self.reactions)
+                        + "\n\n"
+                    )
+            with open(
+                os.path.join(f"{self.name}", "reaction_network.py"),
+                encoding="utf-8",
+                mode="w",
+            ) as f:
+                f.writelines(lines)
         else:
             lines = jl.SET_MODEL.splitlines()
             for line_num, line in enumerate(lines):
@@ -353,9 +373,10 @@ class Text2Model(ReactionRules):
     def _convert_names(
         self,
         line: str,
-        p: List[str] = [],
-        u: List[str] = [],
-        init: List[str] = [],
+        *,
+        p: Optional[List[str]] = None,
+        u: Optional[List[str]] = None,
+        init: Optional[List[str]] = None,
     ) -> str:
         """
         Replace
@@ -365,13 +386,13 @@ class Text2Model(ReactionRules):
 
         Parameters
         ----------
-        p : list of strings (default: [])
+        p : list of strings, optional
             Parameters.
 
-        u : list of strings (default: [])
+        u : list of strings, optional
             Species.
 
-        init : list of strings (default: [])
+        init : list of strings, optional
             Initial conditions.
 
         Returns
@@ -380,6 +401,12 @@ class Text2Model(ReactionRules):
             Each line.
 
         """
+        if p is None:
+            p = []
+        if u is None:
+            u = []
+        if init is None:
+            init = []
         for p_name in p:
             if not p_name.strip() in self.parameters:
                 raise NameError(f"{p_name.strip()} is not defined in model parameters.")
@@ -512,7 +539,7 @@ class Text2Model(ReactionRules):
                         lines[line_num + 3] += (
                             "{}".format(4 * self.indentation)
                             + "self.simulations"
-                            + f"[self.obs_names.index('{desc[0].strip()}'), :, i] = (\n"
+                            + f"[self.obs_names.index('{desc[0].strip()}'), i] = (\n"
                             + f"{5 * self.indentation}"
                             + desc[1].strip(" ").strip()
                         )
@@ -703,7 +730,7 @@ class Text2Model(ReactionRules):
         self.find_cyclic_reaction_routes()
         self._update_parameters()
         self._update_species()
-        self._update_set_model()
+        self._update_diffeq()
         self._update_set_search_param()
         self._update_observable()
         if self.lang == "julia":
@@ -822,6 +849,7 @@ class Text2Model(ReactionRules):
         terminology: Optional[
             Dict[
                 Literal[
+                    "_bind_and_dissociate",
                     "dimerize",
                     "bind",
                     "dissociate",
